package com.mann.dstarlite;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;

import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

public class VisualSearch extends JApplet implements ActionListener {
	
	private static final long serialVersionUID = 1L;
	private JButton setCell;
	private BufferedImage image;
	private JPanel imagePanel;
	
	private GridProblem gp;
	private List<State> curPath;
	private DStarLite dsl;
	
	/*private static final String[] trafficGrid = {
		"                    ",
		"* * * * * *-*-* * * ",
		"------           ---",
		"*   *   *   *   *   ",
		"--*-- *     *  ----*",
		"* * * * * *-*-* * * ",
		"--    ---   --  --  ",
		" * * -- * * -- -* * ",
		"                    ",
		"* * * --- * * -- * *",
		"                    ",
		"*     * * --*-* --- ",
		"--*  -           ---",
		"*   *   * ---   *   ",
		"--*** *     *  ----*",
		"* * * * * *- -*   * ",
		"--    - -   --      ",
		" * * -- * * -- -* * ",
		"                    ",
		"* * * * - * * -- * *"
	};*/
	
	private static final String[] stationaryGrid = {
		"                    ",
		"* * * * * * * * * * ",
		"                    ",
		"*   *   *   *   *   ",
		"  *   *     *      *",
		"* * * * * * * * * * ",
		"                    ",
		" * *    * *     * * ",
		"                    ",
		"* * *     * *    * *",
		"                    ",
		"*     * *   * *     ",
		"  *                 ",
		"*   *   *       *   ",
		"  *** *     *      *",
		"* * * * * *   *   * ",
		"                    ",
		" * *    * *     * * ",
		"                    ",
		"* * * *   * *    * *"
	};
	
	private static final int IMAGE_WIDTH = 801, IMAGE_HEIGHT = 801;
	private static final int CELL_WIDTH = 40, CELL_HEIGHT = 40;
	
	public static final String SET_START = "Start", SET_GOAL = "Goal", SET_WALL = "Wall", SET_WEIGHT = "Weight";
	
	public void init() {
		/* Initialize the problem and the solver. */
		gp = new GridProblem(stationaryGrid);
		dsl = new DStarLite(gp);
		
		image = new BufferedImage(IMAGE_WIDTH, IMAGE_HEIGHT, BufferedImage.TYPE_INT_RGB);
		
		imagePanel = new JPanel(){
			private static final long serialVersionUID = 1L;

			protected void paintComponent(Graphics g) {
				super.paintComponent(g);
				g.drawImage(image, 0, 0, null);
			}
		};
		add(imagePanel, BorderLayout.CENTER);
		
		setCell = new JButton("Set Cell");
		setCell.addActionListener(this);
		JPanel setPanel = new JPanel();
		setPanel.add(setCell, BorderLayout.EAST);
		add(setPanel, BorderLayout.NORTH);
		
		setSize(801, 837);
		setVisible(true);
		drawGrid();
	}
	
	public void computePath() {
		if(gp.getGoal() != null && gp.getStart() != null) {
			curPath = dsl.getShortestPath();
		}
		
		drawGrid();
		this.repaint();
	}
	
	public void drawGrid() {
		Graphics g = image.getGraphics();
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, IMAGE_WIDTH, IMAGE_HEIGHT);
		g.setColor(Color.BLACK);
		
		for( State s : dsl.getExpandedStates()) {
			Cell c = (Cell) s;
			g.setColor(Color.YELLOW);
			g.fillRect(c.getY()*CELL_HEIGHT, c.getX()*CELL_WIDTH, CELL_WIDTH, CELL_HEIGHT);
			g.setColor(Color.BLACK);
		}
		
		if(curPath != null) {
			for(State c : curPath) {
				Cell cell = (Cell) c;
				g.setColor(Color.GREEN);
				g.fillRect(cell.getY()*CELL_HEIGHT, cell.getX()*CELL_WIDTH, CELL_WIDTH, CELL_HEIGHT);
				g.setColor(Color.BLACK);
			}
		}

		Cell[][] cells = gp.getGrid();
		for(int i = 0; i < cells.length; i++) {
			for(int j = 0; j < cells[0].length; j++) {
				Cell c = cells[i][j];
				int xCoord = j*CELL_HEIGHT;
				int yCoord = i*CELL_WIDTH;
				if(c.equals(gp.getStart())) {
					g.setColor(Color.RED);
					g.fillRect(xCoord, yCoord, CELL_WIDTH, CELL_HEIGHT);
				} else if(c.equals(gp.getGoal())) {
					g.setColor(Color.CYAN);
					g.fillRect(xCoord, yCoord, CELL_WIDTH, CELL_HEIGHT);
				} else if(c.isWall()) {
					g.setColor(Color.LIGHT_GRAY);
					g.fillRect(xCoord, yCoord, CELL_WIDTH, CELL_HEIGHT);
				} else if(c.getWeight() > 1){
					int weight = (int)c.getWeight();
					if(weight > 20) {
						weight = 20;
					}
					g.setColor(new Color(150 + weight*5, 50, 50));
					g.fillRect(xCoord, yCoord, CELL_WIDTH, CELL_HEIGHT);
				}
				g.setColor(Color.BLACK);
				g.drawRect(xCoord, yCoord, CELL_WIDTH, CELL_HEIGHT);
				g.drawString(Double.toString((c.getG())), 
						(int)(xCoord + 0.1*CELL_WIDTH), (int)(yCoord + 0.6*CELL_HEIGHT));
			}
		}
	}
	
	public void changeCell(int row, int col, String message) {
		if(message == SET_START) {
			gp.setStart(row, col);
		} else if(message == SET_GOAL) {
			gp.setGoal(row, col);
		} else if(message == SET_WALL) {
			gp.setWall(row, col);
		} else if(message == SET_WEIGHT) {
			gp.setWeight(row, col, 10);
		}
		computePath();
	}
	
	class CustomDialog extends JDialog implements ActionListener, PropertyChangeListener {
		private static final long serialVersionUID = 1L;
		private String typedText = null;
		private JTextField rowField, columnField;
		private JComboBox<String> dropDown;

		private JOptionPane optionPane;

		private String btnString1 = "Enter";
		private String btnString2 = "Cancel";

		/**
		 * Returns null if the typed string was invalid;
		 * otherwise, returns the string as the user entered it.
		 */
		public String getValidatedText() {
			return typedText;
		}

		/** Creates the reusable dialog. */
		public CustomDialog() {
			super();

			setTitle("Set Cell");

			rowField = new JTextField(5);
			columnField = new JTextField(5);
			
			String[] dropDownList = { SET_START, SET_GOAL, SET_WALL, SET_WEIGHT };

			//Create the combo box, and select Start.
			dropDown = new JComboBox<String>(dropDownList);
			dropDown.setSelectedIndex(0);

			//Create an array of the text and components to be displayed.
			String msgString1 = "Enter in the row and column";
			Object[] array = {msgString1, rowField, columnField, dropDown};

			//Create an array specifying the number of dialog buttons
			//and their text.
			Object[] options = {btnString1, btnString2};

			//Create the JOptionPane.
			optionPane = new JOptionPane(array,
					JOptionPane.QUESTION_MESSAGE,
					JOptionPane.YES_NO_OPTION,
					null,
					options,
					options[0]);

			//Make this dialog display it.
			setContentPane(optionPane);

			//Handle window closing correctly.
			setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
			addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent we) {
					/*
					 * Instead of directly closing the window,
					 * we're going to change the JOptionPane's
					 * value property.
					 */
					optionPane.setValue(new Integer(
							JOptionPane.CLOSED_OPTION));
				}
			});

			//Ensure the text field always gets the first focus.
			addComponentListener(new ComponentAdapter() {
				public void componentShown(ComponentEvent ce) {
					rowField.requestFocusInWindow();
				}
			});

			//Register an event handler that puts the text into the option pane.
			rowField.addActionListener(this);
			columnField.addActionListener(this);
			rowField.setText("row");
			rowField.selectAll();

			//Register an event handler that reacts to option pane state changes.
			optionPane.addPropertyChangeListener(this);
			
			this.setSize(300, 200);
		}

		/** This method handles events for the text field. */
		public void actionPerformed(ActionEvent e) {
			optionPane.setValue(btnString1);
		}

		/** This method reacts to state changes in the option pane. */
		public void propertyChange(PropertyChangeEvent e) {
			if (isVisible()) {
				Object value = optionPane.getValue();

				int row = -1, column = -1;
				if (btnString1.equals(value)) {
					try {
						row = Integer.parseInt(rowField.getText());
						column = Integer.parseInt(columnField.getText());
					} catch(Exception e1) {
						e1.printStackTrace();
					}
					clearAndHide();
					VisualSearch.this.changeCell(row, column, (String)dropDown.getSelectedItem());
				} else { //user closed dialog or clicked cancel
					typedText = null;
					clearAndHide();
				}
			}
		}

		/** This method clears the dialog and hides it. */
		public void clearAndHide() {
			rowField.setText(null);
			columnField.setText(null);
			setVisible(false);
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		new CustomDialog().setVisible(true);
	}

}
