package com.mann.jump;

public class JumpPointCell implements Comparable<JumpPointCell> {
	
	private double f, g;
	private int x, y;
	
	private boolean isWall, destWall;
	
	/** Constructs a new JumpPointCell that is initialized as an empty
	 * cell with infinite F and G values.
	 * @param x The x-coordinate of the cell.
	 * @param y The y-coordinate of the cell.
	 */
	public JumpPointCell(int x, int y) {
		this.x = x;
		this.y = y;
		this.isWall = false;
		this.destWall = false;
		this.f = Double.POSITIVE_INFINITY;
		this.g = Double.POSITIVE_INFINITY;
	}
	
	/** We use the same comparator as the A* algorithm. */
	public int Comparable(JumpPointCell other) {
		if(this.f < other.f) {
			return -1;
		} else if(other.getX() == this.x && other.getY() == this.y){
			return 0;
		} else {
			return 1;
		}
	}

	/** Compares F values of each cell.
	 * @param arg0 The cell you want to compare this to.
	 * @return Returns -1 if this cell has a smaller F value, otherwise it
	 * returns 1.
	 */
	public int compareTo(JumpPointCell arg0) {
		try {
			return Comparable((JumpPointCell) arg0);
		} catch(Exception e) {
			return -1;
		}
	}
	
	public boolean equals(JumpPointCell other) {
		return this.x == other.x && this.y == other.y;
	}
	
	public int getX() {
		return this.x;
	}
	
	public int getY() {
		return this.y;
	}
	
	public double getG() {
		return g;
	}
	
	public void setG(double val) {
		this.g = val;
	}
	
	public double getF() {
		return f;
	}
	
	public void setF(double val) {
		this.f = val;
	}
	
	public boolean isWall() {
		return isWall;
	}
	
	public void setWall(boolean val) {
		this.isWall = val;
	}
	
	public String toString() {
		return "<JPC> x:" + this.x + " y:" + this.y;
	}
	
	public int hashCode() {
		return (1000 * x) + y;
	}
	
	public void setDestWall(boolean val) {
		if(this.destWall) {
			setWall(false);
		}
		this.destWall = val;
		if(this.destWall) {
			setWall(true);
		}
	}
	
	public boolean isDestWall() {
		return this.destWall;
	}

}
